@@ -34,6 +34,18 @@ fuzz_types = ["general", "injections", "vulns", "webservices", "stress", "others
 ANOMOLY_THRESHOLD = 1.07142
 RESERVED_PARAMETERS = ["$RESPONSE_ERROR","$CV_EVENT_ID"]
 
+import collections
+def update_nested_dict(dict, value):
+    for k, v in value.items():
+        if isinstance(dict, collections.Mapping):
+            if isinstance(v, collections.Mapping):
+                r = update_nested_dict(dict.get(k, {}), v)
+                dict[k] = r
+            else:
+                dict[k] = value[k]
+        else:
+            dict = {k: value[k]}
+    return dict
 
 def keypaths(nested):
     for key, value in nested.items():
@@ -125,9 +137,12 @@ class CloudvectorDAST(object):
         #             input_json = self._load_input_from_files(file)
         #     self.input_json.update(input_json)
 
+        cv_events = []
+        for _ in ["20"]:
+            print("\n\t ---> Querying for Status code "+str(_)+"*")
+            cv_events += self._process_event_data(apis_to_check, filters_to_query={"http_rsp_status_code":_})
+        print("\n\t\t......done collecting events data from APIShark")
 
-
-        cv_events = self._process_event_data(apis_to_check)
         # if not cv_events:
         #     cv_events = self.prepped_spec
         cv_events += self.prepped_spec
@@ -259,9 +274,9 @@ class CloudvectorDAST(object):
                             x["body"][_.get("name")] = ""
                     #print("this....."+str(_.get("value")), api, method)
                     if _.get("value"):
-                        if type(_.get("value")) is list:
+                        #if type(_.get("value")) is list:
                             #print("its a list....")
-                            self.params_details_from_spec[str(_.get("name"))] = [x["name"] for x in _.get("value")]
+                        self.params_details_from_spec[str(_.get("name"))] = _.get("value")
                         # else:
                         #     print("not list...")
                         #     x["detailed_params"].update(_.get("value"))
@@ -315,7 +330,7 @@ class CloudvectorDAST(object):
         return params_info
 
     def _get_spec_parsed(self, input_path):
-        return self.openapispec_obj.parse_spec(self.apispec_two_path)
+        return self.openapispec_obj.scan_input_spec(self.openapispec_obj.parse_spec(input_path))
 
     def _get_changed_apis(self):
         return self.openapispec_obj.diff
@@ -435,7 +450,7 @@ class CloudvectorDAST(object):
         #                     self.params_captured_in_traffic[statuscode][api][k]=v
         nested_params={}
         more_params = []
-        #print("???"+str(self.params_details_from_spec))
+
         for param, info in self.params_details_from_spec.items():
             #tmp = []
             # if type(info) is list:
@@ -443,9 +458,21 @@ class CloudvectorDAST(object):
             # for k, v in self._recursive_items(info):
             #     if type(v) is not dict:
             #         tmp.append(k)
-            nested_params[param] = info
-            more_params+=info
-        #print("\n\n"+str(nested_params)+"\n\n")
+            _params = []
+            if type(info) is dict:
+                for k, v in keypaths(info):
+                    more_params+=k
+                    _params+=k
+                    info = update_nested_dict(info, {k[-1]: k[-1]})
+            elif type(info) is list:
+                for each in info:
+                    for k, v in keypaths(each):
+                        more_params+=k
+                        _params+=k
+                        each = update_nested_dict(each, {k[-1]:k[-1]})
+            nested_params[param] = _params
+            #more_params+=info
+            more_params = list(set(more_params))
 
         params_for_fixtures = []
         for api,info in self.params_captured_in_traffic.items():
@@ -460,7 +487,6 @@ class CloudvectorDAST(object):
                                 if _ not in params_for_fixtures and _ not in RESERVED_PARAMETERS:
                                     params_for_fixtures.append(_)
 
-
         with open("tests/conftest.py", 'w+') as fh:
             fh.write(template.render(api_info=params_for_fixtures, api_detailed_info=self.params_details_from_spec,
                                      nested_params=nested_params, AUTH_API=auth_api, more_params=more_params,
@@ -499,7 +525,9 @@ class CloudvectorDAST(object):
     def _create_assertions_map(self, url, params, req_payload, resp_payload):
         assertions_map = {}
         if url not in self.assertions_map:
-            url = url.replace("//","/").replace("/", "_").replace("-", "_").replace("{", "").replace("}", "").split("?")[0]
+            url = url.replace("//","/").replace("/", "_S_").replace("{", "").replace("}", "").replace(" ","").split("?")[0]
+            if url.startswith("_S_"):
+                url = "_"+url[3:]
             self.assertions_map[url] = {}
         if None in params:
             params.remove(None)
@@ -550,13 +578,14 @@ class CloudvectorDAST(object):
         for _ in cv_events:
             #print(_)
             event = _[0]
+            rsp_codes_in_traffic = {}
             if _[1]:
                 if int(_[1]["attributes"]["http_rsp_status_code"]) not in range(200, 299):
                     continue
             if event["url"]:
                 # api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
                 #     0]
-                api = str(event["http-req-url"]).lstrip("/").replace("/", "_").replace("-", "_").split("?")[0]
+                api = str(event["http-req-url"]).lstrip("/").replace("/", "_S_").replace(" ","").split("?")[0]
                 if api not in apis_to_be_tested:
                     apis_to_be_tested[api] = {}
                 #apis_to_be_tested[api]["method"] = event["method"]
@@ -661,13 +690,15 @@ class CloudvectorDAST(object):
         #                                     event["rsp_body"])
 
         for _ in cv_events:
+            #print(_)
             event = _[0]
             if _[1]:
-                if _[1]["attributes"]["http_rsp_status_code"] not in ["200", "201"]:
+                if int(_[1]["attributes"]["http_rsp_status_code"]) not in range(200, 299):
                     continue
             if event["url"]:
-                api = str(event["http-req-url"]).lstrip("/").replace("/", "_").replace("-", "_").split("?")[
-                    0]
+                # api = str(event["http-req-url"]).lstrip("/").rstrip("/").replace("/", "_").replace("-", "_").split("?")[
+                #     0]
+                api = str(event["http-req-url"]).lstrip("/").replace("/", "_S_").replace(" ","").split("?")[0]
                 if api not in apis_to_be_tested:
                     apis_to_be_tested[api] = {}
                 #apis_to_be_tested[api]["method"] = event["method"]
@@ -686,13 +717,22 @@ class CloudvectorDAST(object):
                     known_params = apis_to_be_tested[api][method].get("params")
                 else:
                     known_params = []
-                apis_to_be_tested[api][method]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"),{}).get(method,{})
+                apis_to_be_tested[api][method]["params"] = new_spec_info.get(str(event["http-req-url"]).replace("//", "/"),{}).get(method,[])
+
+                params_in_traffic = event.get("body",{}).keys()
+
+                for _ in params_in_traffic:
+                    if _ not in apis_to_be_tested[api][method]["params"]:
+                        apis_to_be_tested[api][method]["params"].append(_)
+
                 if apis_to_be_tested[api][method]["params"] is None:
                     apis_to_be_tested[api][method]["params"] = []
+
                 apis_to_be_tested[api][method]["params"] = list(set(apis_to_be_tested[api][method]["params"]))
                 apis_to_be_tested[api][method]["params"].append("host")
                 apis_to_be_tested[api][method]["params"].append("url_prefix")
                 apis_to_be_tested[api][method]["params"].append("access_token")
+
                 host_url = str(event["host"]).lower()
                 hosts.append(host_url)
                 self._create_assertions_map(event["http-req-url"], apis_to_be_tested[api][method]["params"], event["body"],
@@ -753,12 +793,12 @@ class CloudvectorDAST(object):
         print(response.text)
         return response.json()
 
-    def _process_event_data(self, apis_to_check=[]):
+    def _process_event_data(self, apis_to_check=[], filters_to_query={}):
         print("\n\ncollecting events data from APIShark....")
         cv_requests = []
-        events = self.ceobj.get_all_raw_events(apis_to_check)
-        print("\nNo. of events from APIShark being processed: " + str(len(events)))
+        events = self.ceobj.get_all_raw_events(apis_to_check, **filters_to_query)
         print("\n\nprocessing events data from APIShark....")
+        processed_events = 0
         for event in events:
             if "http-req-header-Cv-Fuzzed-Event" in event["attributes"]["event_json"]:
                 continue
@@ -879,7 +919,8 @@ class CloudvectorDAST(object):
                 #     pass
             #print(self.params_captured_in_traffic)
             cv_requests.append([request, event])
-        print("\n\t\t......done collecting events data from APIShark")
+            processed_events+=1
+        print("\nNo. of events processed from Cloudvector Enterprise Dashboard: " + str(processed_events))
         return cv_requests
 
 
@@ -941,6 +982,7 @@ def main():
         with open(os.path.join(os.getcwd(), "my_cesetup.yaml"), "w+") as fobj:
             yaml.dump({"ce_host": str(ce_host), "ce_username": str(ce_username)}, fobj)
     enable_fuzzing = False
+
     if sys.argv[1:]:
         if sys.argv[1:][0] == "--fuzz":
             print("\nFuzzing enabled!\n")
